<?php

namespace Drupal\entity_track;

use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Plugin\DefaultPluginManager;

/**
 * Manages entity track plugins.
 *
 * @method \Drupal\entity_track\EntityTrackInterface createInstance($plugin_id, array $configuration = [])
 */
class EntityTrackPluginManager extends DefaultPluginManager {

  /**
   * Constructs a new EntityTrackPluginManager.
   *
   * @param \Traversable $namespaces
   *   An object that implements \Traversable which contains the root paths
   *   keyed by the corresponding namespace to look for plugin implementations.
   * @param \Drupal\Core\Cache\CacheBackendInterface $cache_backend
   *   Cache backend instance to use.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler.
   */
  public function __construct(\Traversable $namespaces, CacheBackendInterface $cache_backend, ModuleHandlerInterface $module_handler) {
    parent::__construct('Plugin/EntityTrack/Track', $namespaces, $module_handler, 'Drupal\entity_track\EntityTrackInterface', 'Drupal\entity_track\Annotation\EntityTrack');
    $this->alterInfo('entity_track_info');
    $this->setCacheBackend($cache_backend, 'entity_track_plugins');
  }

}
